-- FUN/OPTIONAL Addon
-- Most of the code is from the GenderTooltip addon. Huge thanks! Go download it if you wish. 

-- Localization
local L = GenderTooltip_LocalizationTable;

-- AddOn options
GenderTooltip_Options_Defaults = {
	Enabled = true,
	TextColoring = false,
	Colors = {
		None = "FC1FF",
		Male = "69CCF0",
		Female = "F58CBA",
	};
};
GenderTooltip_Options = GenderTooltip_Options_Defaults;

-- Utility function to convert RGB to their hex equivelents
local function RGBToHex(r, g, b)
	r = tonumber(r);
	g = tonumber(g);
	b = tonumber(b);
	if r < 0 then
		r = 0;
	elseif r > 255 then
		while r > 255 do
			r = r - 255;
		end
	end
	if g < 0 then
		g = 0;
	elseif g > 255 then
		while g > 255 do
			g = g - 255;
		end
	end
	if b < 0 then
		b = 0;
	elseif b > 255 then
		while b > 255 do
			b = b - 255;
		end
	end
	return string.format("%02X%02X%02X", r, g, b)
end

-- Opposite of RGBToHex
local function HexToRGB(hex)
	hex = hex:gsub("#","");
	return tonumber("0x"..hex:sub(1, 2)), tonumber("0x"..hex:sub(3, 4)), tonumber("0x"..hex:sub(5, 6));
end

-- Convert RGB to Floats
local function RGBToFloat(r, g, b)
	local base = 1 / 255;
	return base * r, base * g, base * b;
end

-- Convert Floats to RGB
local function FloatToRGB(r, g, b)
	local base = 1 / 255;
	return r / base, g / base, b / base;
end

-- Options Panel
GenderTooltipOptionsPanel = CreateFrame("Frame", "GenderTooltipOptionsPanel", UIParent);
GenderTooltipOptionsPanel.name = "LB Player Gender";

-- Create the title
GenderTooltipOptionsPanel.titleText = GenderTooltipOptionsPanel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
GenderTooltipOptionsPanel.titleText:SetPoint("TOPLEFT", 16, -16);
GenderTooltipOptionsPanel.titleText:SetText(GenderTooltipOptionsPanel.name);

-- Create the "Enabled" checkbox
GenderTooltipOptionsPanel.enabledCheckbox = CreateFrame("CheckButton", "GenderTooltipOptionsPanelEnabledCheckBox", GenderTooltipOptionsPanel, "InterfaceOptionsCheckButtonTemplate");
GenderTooltipOptionsPanel.enabledCheckbox:SetPoint("TOPLEFT", GenderTooltipOptionsPanel.titleText, "BOTTOMLEFT", -2, -8);
_G[GenderTooltipOptionsPanel.enabledCheckbox:GetName().."Text"]:SetText(L["Enabled"]);

-- Create the "Color Text" checkbox
GenderTooltipOptionsPanel.colorCheckbox = CreateFrame("CheckButton", "GenderTooltipOptionsPanelColorCheckBox", GenderTooltipOptionsPanel, "InterfaceOptionsCheckButtonTemplate");
GenderTooltipOptionsPanel.colorCheckbox:SetPoint("TOPLEFT", GenderTooltipOptionsPanel.enabledCheckbox, "BOTTOMLEFT", 0, -8);
_G[GenderTooltipOptionsPanel.colorCheckbox:GetName().."Text"]:SetText(L["ColorText"]);

-- Coloring function for Male/Female
GenderTooltipOptionsPanel.OpenColorPicker = function(texture)
	if not texture then return end
	local info = {};
	info.swatchFunc = function()
		texture:SetVertexColor(ColorPickerFrame:GetColorRGB());
	end
	info.hasOpacity = false;
	info.previousValues = texture:GetVertexColor();
	info.cancelFunc = function()
		texture:SetVertexColor(ColorPicker_GetPreviousValues());
	end
	info.r, info.g, info.b = texture:GetVertexColor();
	OpenColorPicker(info);
end

-- Create our "Male" button
GenderTooltipOptionsPanel.maleColorBlock = CreateFrame("CheckButton", "GenderTooltipOptionsPanelMaleColorCheckBox", GenderTooltipOptionsPanel, "InterfaceOptionsCheckButtonTemplate");
GenderTooltipOptionsPanel.maleColorBlock:SetSize(16, 16);
GenderTooltipOptionsPanel.maleColorBlock:SetPoint("TOPLEFT", GenderTooltipOptionsPanel.colorCheckbox, "BOTTOMLEFT", 4, -8);
GenderTooltipOptionsPanel.maleColorBlock.texture = GenderTooltipOptionsPanel.maleColorBlock:CreateTexture(nil, "OVERLAY", nil, -7);
GenderTooltipOptionsPanel.maleColorBlock.texture:SetAllPoints(GenderTooltipOptionsPanel.maleColorBlock);
GenderTooltipOptionsPanel.maleColorBlock.texture:SetColorTexture(1, 1, 1);
GenderTooltipOptionsPanel.maleColorBlock.texture:SetVertexColor(RGBToFloat(HexToRGB(GenderTooltip_Options.Colors["Male"])));
GenderTooltipOptionsPanel.maleColorBlock:EnableMouse(true);
GenderTooltipOptionsPanel.maleColorBlock:SetNormalTexture(GenderTooltipOptionsPanel.maleColorBlock.texture);
GenderTooltipOptionsPanel.maleColorBlock:Disable();
_G[GenderTooltipOptionsPanel.maleColorBlock:GetName().."Text"]:SetText(" "..L["Male"]);
GenderTooltipOptionsPanel.maleColorBlock:SetScript("OnMouseDown", function(self, button, ...)
	if button == "LeftButton" and GenderTooltipOptionsPanel.colorCheckbox:GetChecked() then
		GenderTooltipOptionsPanel.OpenColorPicker(self.texture);
	end
end);

-- Create our "Female" button
GenderTooltipOptionsPanel.femaleColorBlock = CreateFrame("CheckButton", "GenderTooltipOptionsPanelFemaleColorCheckBox", GenderTooltipOptionsPanel, "InterfaceOptionsCheckButtonTemplate");
GenderTooltipOptionsPanel.femaleColorBlock:SetSize(16, 16);
GenderTooltipOptionsPanel.femaleColorBlock:SetPoint("TOPLEFT", GenderTooltipOptionsPanel.maleColorBlock, "BOTTOMLEFT", 0, -8);
GenderTooltipOptionsPanel.femaleColorBlock.texture = GenderTooltipOptionsPanel.femaleColorBlock:CreateTexture(nil, "OVERLAY", nil, -7);
GenderTooltipOptionsPanel.femaleColorBlock.texture:SetAllPoints(GenderTooltipOptionsPanel.femaleColorBlock);
GenderTooltipOptionsPanel.femaleColorBlock.texture:SetColorTexture(1, 1, 1);
GenderTooltipOptionsPanel.femaleColorBlock.texture:SetVertexColor(RGBToFloat(HexToRGB(GenderTooltip_Options.Colors["Female"])));
GenderTooltipOptionsPanel.femaleColorBlock:EnableMouse(true);
GenderTooltipOptionsPanel.femaleColorBlock:SetNormalTexture(GenderTooltipOptionsPanel.femaleColorBlock.texture);
GenderTooltipOptionsPanel.femaleColorBlock:Disable();
_G[GenderTooltipOptionsPanel.femaleColorBlock:GetName().."Text"]:SetText(" "..L["Female"]);
GenderTooltipOptionsPanel.femaleColorBlock:SetScript("OnMouseDown", function(self, button, ...)
	if button == "LeftButton" and GenderTooltipOptionsPanel.colorCheckbox:GetChecked() then
		GenderTooltipOptionsPanel.OpenColorPicker(self.texture);
	end
end);

-- Visually Enable/Disable the Male/Female buttons
GenderTooltipOptionsPanel.colorCheckbox:SetScript("OnMouseUp", function(self, button, ...)
	if button == "LeftButton" then
		if self:GetChecked() then -- This is the previous state, because reasons
			GenderTooltipOptionsPanel.maleColorBlock:SetAlpha(0.5);
			GenderTooltipOptionsPanel.femaleColorBlock:SetAlpha(0.5);
		else
			GenderTooltipOptionsPanel.maleColorBlock:SetAlpha(1.0);
			GenderTooltipOptionsPanel.femaleColorBlock:SetAlpha(1.0);
		end
	end
end);

-- Panel callbacks
GenderTooltipOptionsPanel.okay = function()
	GenderTooltip_Options.Enabled = GenderTooltipOptionsPanel.enabledCheckbox:GetChecked();
	GenderTooltip_Options.TextColoring = GenderTooltipOptionsPanel.colorCheckbox:GetChecked();
	GenderTooltip_Options.Colors["Male"] = RGBToHex(FloatToRGB(GenderTooltipOptionsPanel.maleColorBlock.texture:GetVertexColor()));
	GenderTooltip_Options.Colors["Female"] = RGBToHex(FloatToRGB(GenderTooltipOptionsPanel.femaleColorBlock.texture:GetVertexColor()));
end
GenderTooltipOptionsPanel.cancel = function()
	GenderTooltipOptionsPanel.enabledCheckbox:SetChecked(GenderTooltip_Options.Enabled);
	GenderTooltipOptionsPanel.colorCheckbox:SetChecked(GenderTooltip_Options.TextColoring);
	if GenderTooltip_Options.TextColoring == false then
		GenderTooltipOptionsPanel.maleColorBlock:SetAlpha(0.5);
		GenderTooltipOptionsPanel.femaleColorBlock:SetAlpha(0.5);
	else
		GenderTooltipOptionsPanel.maleColorBlock:SetAlpha(1.0);
		GenderTooltipOptionsPanel.femaleColorBlock:SetAlpha(1.0);
	end
	GenderTooltipOptionsPanel.maleColorBlock.texture:SetVertexColor(RGBToFloat(HexToRGB(GenderTooltip_Options.Colors["Male"])));
	GenderTooltipOptionsPanel.femaleColorBlock.texture:SetVertexColor(RGBToFloat(HexToRGB(GenderTooltip_Options.Colors["Female"])));
end
GenderTooltipOptionsPanel.refresh = GenderTooltipOptionsPanel.cancel;
GenderTooltipOptionsPanel.default = function()
	GenderTooltip_Options = GenderTooltip_Options_Defaults;
end

-- Add our Options Panel
InterfaceOptions_AddCategory(GenderTooltipOptionsPanel);

-- Handler function for the slash commands
local function GenderTooltip_SlashCommand(msg, editBox)
	msg = msg:lower();
	local args = {};
	for word in msg:gmatch("[^%s]+") do
		table.insert(args, word);
	end

	if args[1] then
		if arg[1] == L["help"] then
			print(L["Error_Usage"]);
		elseif args[1] == L["on"] then
			GenderTooltip_Options.Enabled = true;
		elseif args[1] == L["off"] then
			GenderTooltip_Options.Enabled = false;
		elseif args[1] == L["color"] then
			if args[2] == L["on"] then
				GenderTooltip_Options.TextColoring = true;
			elseif args[2] == L["off"] then
				GenderTooltip_Options.TextColoring = false;
			elseif args[2] == L["set"] then
				local gender = "None";
				if args[3] == L["Male"]:lower() then
					gender = "Male";
				elseif args[3] == L["Female"]:lower() then
					gender = "Female";
				end

				if not args[4] or not args[5] or not args[6] then
					print(L["Error_Color_set"]);
				else
					GenderTooltip_Options.Colors[gender] = RGBToHex(args[4], args[5], args[6]);
				end
			else
				print(L["Error_Color"]);
			end
		else
			print(L["Error_Unk_Args"]);
		end
	else
		-- Twice to open to our category, first only seems to open the Options window sometimes
		InterfaceOptionsFrame_OpenToCategory(GenderTooltipOptionsPanel);
		InterfaceOptionsFrame_OpenToCategory(GenderTooltipOptionsPanel);
	end
end
SlashCmdList["GENDERTOOLTIP"] = GenderTooltip_SlashCommand;
SLASH_GENDERTOOLTIP1 = "/gto";
SLASH_GENDERTOOLTIP2 = "/gtto";
SLASH_GENDERTOOLTIP3 = "/gendertooltipoptions";

-- Handle when the user mouses over a unit, replacing the "Level" line.
local function GenderTooltip_OnSetUnit(self)
	if GenderTooltip_Options.Enabled == false then
		return nil;
	end

	local genders = {"", L["Male"], L["Female"]};
	local gender_options = {"None", "Male", "Female"};
	local unit = select(2, self:GetUnit());
	if not unit then
		return nil;
	end

	if UnitIsPlayer(unit) then
		for i = 2, self:NumLines() do
			if _G["GameTooltipTextLeft"..i]:GetText():find("^" .. L["Level"]) then
				-- Bugfix from 'Richard' on Curse.com for level display
				-- "Made a few changes to accomodate opposite faction players who's level shows up as ??."
				local level = UnitLevel(unit);
				if level == -1 then
					level = "??";
				end
				local gender = genders[UnitSex(unit)];
				if GenderTooltip_Options.TextColoring == true then
					gender = "|cFF"..GenderTooltip_Options.Colors[gender_options[UnitSex(unit)]]..gender.."|r";
				end
				local race = UnitRace(unit);
				local class = UnitClass(unit);
				local txt = L["Level"].." "..level.." "..gender;

				if race ~= nil then
					txt = txt.." "..race;
				end
				if class ~= nil then
					txt = txt.." "..class;
				end;

				_G["GameTooltipTextLeft"..i]:SetText(txt.." ("..L["Player"]..")");

				self:Show();
				break;
			end
		end
	end
end
GameTooltip:HookScript("OnTooltipSetUnit", GenderTooltip_OnSetUnit);
